@@ -14,7 +14,7 @@ useful for solving the Assignment Problem.
 Assignment Problem
 ==================
 
-Let *C* be an *n*\ x\ *n* matrix representing the costs of each of *n* workers
+Let *C* be an *n* by *n* matrix representing the costs of each of *n* workers
 to perform any of *n* jobs. The assignment problem is to assign jobs to
 workers in a way that minimizes the total cost. Since each worker can perform
 only one job and each job can be assigned to only one worker the assignments
@@ -23,13 +23,13 @@ represent an independent set of the matrix *C*.
 One way to generate the optimal set is to create all permutations of
 the indexes necessary to traverse the matrix so that no row and column
 are used more than once. For instance, given this matrix (expressed in
-Python)::
+Python):
 
     matrix = [[5, 9, 1],
               [10, 3, 2],
               [8, 7, 4]]
 
-You could use this code to generate the traversal indexes::
+You could use this code to generate the traversal indexes:
 
     def permute(a, results):
         if len(a) == 1:
@@ -48,7 +48,7 @@ You could use this code to generate the traversal indexes::
     results = []
     permute(range(len(matrix)), results) # [0, 1, 2] for a 3x3 matrix
 
-After the call to permute(), the results matrix would look like this::
+After the call to permute(), the results matrix would look like this:
 
     [[0, 1, 2],
      [0, 2, 1],
@@ -58,13 +58,12 @@ After the call to permute(), the results matrix would look like this::
      [2, 1, 0]]
 
 You could then use that index matrix to loop over the original cost matrix
-and calculate the smallest cost of the combinations::
+and calculate the smallest cost of the combinations:
 
-    n = len(matrix)
     minval = sys.maxsize
-    for row in range(n):
+    for indexes in results:
         cost = 0
-        for col in range(n):
+        for row, col in enumerate(indexes):
             cost += matrix[row][col]
         minval = min(cost, minval)
 
@@ -85,20 +84,20 @@ This version is based on
 http://csclab.murraystate.edu/~bob.pilgrim/445/munkres.html
 
 This version was written for Python by Brian Clapper from the algorithm
-at the above web site. (The ``Algorithm::Munkres`` Perl version, in CPAN, was
+at the above web site. (The ``Algorithm:Munkres`` Perl version, in CPAN, was
 clearly adapted from the same web site.)
 
 Usage
 =====
 
-Construct a Munkres object::
+Construct a Munkres object:
 
     from munkres import Munkres
 
     m = Munkres()
 
 Then use it to compute the lowest cost assignment from a cost matrix. Here's
-a sample program::
+a sample program:
 
     from munkres import Munkres, print_matrix
 
@@ -115,7 +114,7 @@ a sample program::
         print '(%d, %d) -> %d' % (row, column, value)
     print 'total cost: %d' % total
 
-Running that program produces::
+Running that program produces:
 
     Lowest cost through this matrix:
     [5, 9, 1]
@@ -152,7 +151,7 @@ the combination of elements (one from each row and column) that results in
 the smallest cost. It's also possible to use the algorithm to maximize
 profit. To do that, however, you have to convert your profit matrix to a
 cost matrix. The simplest way to do that is to subtract all elements from a
-large value. For example::
+large value. For example:
 
     from munkres import Munkres, print_matrix
 
@@ -177,7 +176,7 @@ large value. For example::
 
     print 'total profit=%d' % total
 
-Running that program produces::
+Running that program produces:
 
     Highest profit through this matrix:
     [5, 9, 1]
@@ -189,27 +188,32 @@ Running that program produces::
     total profit=23
 
 The ``munkres`` module provides a convenience method for creating a cost
-matrix from a profit matrix. Since it doesn't know whether the matrix contains
-floating point numbers, decimals, or integers, you have to provide the
+matrix from a profit matrix. By default, it calculates the maximum profit
+and subtracts every profit from it to obtain a cost. If, however, you
+need a more general function, you can provide the
 conversion function; but the convenience method takes care of the actual
-creation of the cost matrix::
+creation of the matrix:
 
     import munkres
 
-    cost_matrix = munkres.make_cost_matrix(matrix,
-                                           lambda cost: sys.maxsize - cost)
+    cost_matrix = munkres.make_cost_matrix(
+        matrix,
+        lambda profit: 1000.0 - math.sqrt(profit))
 
-So, the above profit-calculation program can be recast as::
+So, the above profit-calculation program can be recast as:
 
     from munkres import Munkres, print_matrix, make_cost_matrix
 
     matrix = [[5, 9, 1],
               [10, 3, 2],
               [8, 7, 4]]
-    cost_matrix = make_cost_matrix(matrix, lambda cost: sys.maxsize - cost)
+    cost_matrix = make_cost_matrix(matrix)
+    # cost_matrix == [[5, 1, 9],
+    #                 [0, 7, 8],
+    #                 [2, 3, 6]]
     m = Munkres()
     indexes = m.compute(cost_matrix)
-    print_matrix(matrix, msg='Lowest cost through this matrix:')
+    print_matrix(matrix, msg='Highest profits through this matrix:')
     total = 0
     for row, column in indexes:
         value = matrix[row][column]
@@ -232,7 +236,7 @@ Simply use the munkres.DISALLOWED constant.
                                           (cost != DISALLOWED) else DISALLOWED)
     m = Munkres()
     indexes = m.compute(cost_matrix)
-    print_matrix(matrix, msg='Lowest cost through this matrix:')
+    print_matrix(matrix, msg='Highest profit through this matrix:')
     total = 0
     for row, column in indexes:
         value = matrix[row][column]
@@ -306,10 +310,10 @@ __all__     = ['Munkres', 'make_cost_matrix', 'DISALLOWED']
 # ---------------------------------------------------------------------------
 
 # Info about the module
-__version__   = "1.0.10"
+__version__   = "1.0.12"
 __author__    = "Brian Clapper, bmc@clapper.org"
 __url__       = "http://software.clapper.org/munkres/"
-__copyright__ = "(c) 2008 Brian M. Clapper"
+__copyright__ = "(c) 2008-2017 Brian M. Clapper"
 __license__   = "Apache Software License"
 
 # Constants
@@ -473,7 +477,14 @@ class Munkres:
         C = self.C
         n = self.n
         for i in range(n):
-            minval = min(self.C[i])
+            vals = [x for x in self.C[i] if x is not DISALLOWED]
+            if len(vals) == 0:
+                # All values in this row are DISALLOWED. This matrix is
+                # unsolvable.
+                raise UnsolvableMatrix(
+                    "Row {0} is entirely DISALLOWED.".format(i)
+                )
+            minval = min(vals)
             # Find the minimum value for this row and subtract that minimum
             # from every element in the row.
             for j in range(n):
@@ -532,11 +543,11 @@ class Munkres:
         """
         step = 0
         done = False
-        row = -1
-        col = -1
+        row = 0
+        col = 0
         star_col = -1
         while not done:
-            (row, col) = self.__find_a_zero()
+            (row, col) = self.__find_a_zero(row, col)
             if row < 0:
                 done = True
                 step = 6
@@ -622,20 +633,21 @@ class Munkres:
         for i in range(self.n):
             for j in range(self.n):
                 if (not self.row_covered[i]) and (not self.col_covered[j]):
-                    if minval > self.C[i][j]:
+                    if self.C[i][j] is not DISALLOWED and minval > self.C[i][j]:
                         minval = self.C[i][j]
         return minval
 
-    def __find_a_zero(self):
+
+    def __find_a_zero(self, i0=0, j0=0):
         """Find the first uncovered element with value 0"""
         row = -1
         col = -1
-        i = 0
+        i = i0
         n = self.n
         done = False
 
         while not done:
-            j = 0
+            j = j0
             while True:
                 if (self.C[i][j] == 0) and \
                         (not self.row_covered[i]) and \
@@ -643,11 +655,11 @@ class Munkres:
                     row = i
                     col = j
                     done = True
-                j += 1
-                if j >= n:
+                j = (j + 1) % n
+                if j == j0:
                     break
-            i += 1
-            if i >= n:
+            i = (i + 1) % n
+            if i == i0:
                 done = True
 
         return (row, col)
@@ -715,23 +727,24 @@ class Munkres:
 # Functions
 # ---------------------------------------------------------------------------
 
-def make_cost_matrix(profit_matrix, inversion_function):
+def make_cost_matrix(profit_matrix, inversion_function=None):
     """
     Create a cost matrix from a profit matrix by calling
     'inversion_function' to invert each value. The inversion
     function must take one numeric argument (of any type) and return
     another numeric argument which is presumed to be the cost inverse
-    of the original profit.
+    of the original profit. In case the inversion function is not provided,
+    calculate it as max(matrix) - matrix.
 
     This is a static method. Call it like this:
 
-    .. python::
+    .. python:
 
         cost_matrix = Munkres.make_cost_matrix(matrix, inversion_func)
 
     For example:
 
-    .. python::
+    .. python:
 
         cost_matrix = Munkres.make_cost_matrix(matrix, lambda x : sys.maxsize - x)
 
@@ -740,11 +753,16 @@ def make_cost_matrix(profit_matrix, inversion_function):
             The matrix to convert from a profit to a cost matrix
 
         inversion_function : function
-            The function to use to invert each entry in the profit matrix
+            The function to use to invert each entry in the profit matrix.
+            In case it is not provided, calculate it as max(matrix) - matrix.
 
     :rtype: list of lists
     :return: The converted matrix
     """
+    if not inversion_function:
+      maximum = max(max(row) for row in profit_matrix)
+      inversion_function = lambda x: maximum - x
+
     cost_matrix = []
     for row in profit_matrix:
         cost_matrix.append([inversion_function(value) for value in row])
